
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: ChatFrame_AddMessageEventFilter, ChatFrame_RemoveMessageEventFilter, DEFAULT_CHAT_FRAME, GetLocale

local next = next
local type = type

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat Filters")

local ADDON_PATTERNS = {
	-- BadBoy
	"^|cFF33FF99BadBoy|r",

	-- Blizzard
	"^Loot Specialization set to: ",
	"^Dungeon Difficulty set to ",
	"^Raid Difficulty set to ",
	"^OnDatabaseShutdown table: ",

	-- Dominos
	"|cff33ff99Dominos|r:",

	-- GTFO
	"^%[GTFO%]",

	-- Gnosis
	"^|cff33ff99Gnosis|r:",

	-- TomCats
	"^|TInterface/AddOns/TomCats/images/tomcat_chat_icon_lg.blp:",
	"^Please be patient while I continuously update this addon in the upcoming days! Thank you for supporting TomCat's Tours <3$",

	-- VuhDo
	"^|cffffe566{VuhDo}|r",
}

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function FilterAddOn(text)
	if type(text) == "string" then
		for _, pattern in next, ADDON_PATTERNS do
			if text:find(pattern) then
				return true
			end
		end
	end

	return false
end

local function FilterForeign(self, event, text)
	if text:find("[\228-\233]") then
		return true
	end
end

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	self:EnableAddOnFilter()
	self:ScheduleTimer("DisableAddOnFilter", 30)
end

function M:OnEnable()
	if (C.PLAYER_REGION_ID ~= 1) then return end
	self:EnableForeignFilter()
end

function M:OnDisable()
	if (C.PLAYER_REGION_ID ~= 1) then return end
	self:DisableForeignFilter()
end

function M:EnableAddOnFilter()
	if not self:IsHooked(DEFAULT_CHAT_FRAME, "AddMessage") then
		self:SecureHook(DEFAULT_CHAT_FRAME, "AddMessage", "ChatFrame_AddMessage")
	end

	if not self:IsHooked(DEFAULT_CHAT_FRAME, "BackFillMessage") then
		self:SecureHook(DEFAULT_CHAT_FRAME, "BackFillMessage", "ChatFrame_AddMessage")
	end
end

function M:DisableAddOnFilter()
	self:Unhook(DEFAULT_CHAT_FRAME, "AddMessage")
	self:Unhook(DEFAULT_CHAT_FRAME, "BackFillMessage")
end

function M:EnableForeignFilter()
	if GetLocale() ~= "enGB" and GetLocale() ~= "enUS" then return end
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", FilterForeign)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", FilterForeign)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", FilterForeign)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", FilterForeign)
end

function M:DisableForeignFilter()
	if GetLocale() ~= "enGB" and GetLocale() ~= "enUS" then return end
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_CHANNEL", FilterForeign)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SAY", FilterForeign)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", FilterForeign)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_YELL", FilterForeign)
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:ChatFrame_AddMessage(chatFrame, text1, ...)
	if FilterAddOn(text1) then
		chatFrame:RemoveMessagesByPredicate(function (text2) return text1 == text2 end)
	end
end
